@@ -10,6 +10,10 @@ from .utils import read_pvsyst_file
 from tyba_client.utils import string_enum
 
 
+class ValidationError(ValueError):
+    pass
+
+
 def opt_field():
     return field(default=None, metadata=config(exclude=lambda x: x is None))
 
@@ -252,14 +256,14 @@ class PVModuleMermoudLejeune(object):
 @dataclass_json
 @dataclass
 class Transformer(object):
-    rating: float
     load_loss: float
     no_load_loss: float
+    rating: t.Optional[float] = opt_field()
 
 
 @dataclass_json
 @dataclass
-class DCLosses(object):
+class _DCLosses(object):
     dc_optimizer: t.Optional[float] = opt_field()
     enable_snow_model: t.Optional[bool] = opt_field()
     dc_wiring: t.Optional[float] = opt_field()
@@ -282,11 +286,20 @@ class ACLosses(object):
     transmission: t.Optional[float] = opt_field()
     poi_adjustment: t.Optional[float] = opt_field()
     mv_transformer: t.Optional[Transformer] = opt_field()
+    hv_transformer: t.Optional[Transformer] = opt_field()
+
+    def __post_init__(self):
+        if not (
+            (self.transformer_load is None and self.transformer_no_load is None) or
+            self.hv_transformer is None
+        ):
+            raise ValidationError(
+                "Cannot provide hv_transformer if transformer_load or transformer_no_load are provided")
 
 
 @dataclass_json
 @dataclass
-class Losses(DCLosses, ACLosses):
+class Losses(_DCLosses, ACLosses):
     pass
 
 
@@ -367,6 +380,12 @@ _psm_data_map = {
 }
 
 
+@string_enum
+class ArrayDegradationMode(str, Enum):
+    linear = "linear"
+    compounding = "compounding"
+
+
 @dataclass_json
 @dataclass
 class PVModel(object):
@@ -377,7 +396,8 @@ class PVModel(object):
     losses: t.Optional[Losses] = opt_field()
     layout: t.Optional[Layout] = opt_field()
     project_term: t.Optional[int] = None
-    dc_degradation: t.Optional[float] = None
+    array_degradation_rate: t.Optional[float] = None
+    array_degradation_mode: t.Optional[ArrayDegradationMode] = "linear"
 
 
 PVGenerationModel = DetailedPVModel = PVModel
@@ -413,7 +433,7 @@ class DCExternalGenerationModel(object):
     production_override: DCProductionProfile
     system_design: BaseSystemDesign
     inverter: t.Union[Inverter, ONDInverter, str]
-    losses: DCLosses = opt_field()
+    losses: ACLosses = opt_field()
     project_term: t.Optional[int] = None
 
 
@@ -431,6 +451,18 @@ class BatteryHVAC(object):
     aux_xfmr_efficiency: float
 
 
+@dataclass_json
+@dataclass
+class TableCapDegradationModel(object):
+    annual_capacity_derates: t.List[float]
+
+
+@dataclass_json
+@dataclass
+class TableEffDegradationModel(object):
+    annual_efficiency_derates: t.List[float]
+
+
 @dataclass_json
 @dataclass
 class Battery(object):
@@ -438,10 +470,12 @@ class Battery(object):
     energy_capacity: float
     charge_efficiency: float
     discharge_efficiency: float
-    degradation_rate: float
-    term: int
+    term: float
+    degradation_rate: t.Optional[float] = opt_field()
     degradation_annual_cycles: t.Optional[float] = 365
     hvac: t.Optional[BatteryHVAC] = opt_field()
+    capacity_degradation_model: t.Optional[TableCapDegradationModel] = opt_field()
+    efficiency_degradation_model: t.Optional[TableEffDegradationModel] = opt_field()
 
 
 @dataclass_json
@@ -456,6 +490,8 @@ class StorageInputs(object):
     dart: t.Optional[bool] = opt_field()
     dam_annual_cycle_limit: t.Optional[float] = opt_field()
     no_virtual_trades: t.Optional[bool] = opt_field()
+    initial_soe: t.Optional[float] = 0
+    symmetric_reg: t.Optional[bool] = False
 
 
 @dataclass_json
@@ -477,12 +513,22 @@ class Utilization(object):
     lower: float
     upper: float
 
+
+@dataclass_json
+@dataclass
+class TimeSeriesUtilization(object):
+    actual: t.List[float]
+    lower: t.List[float]
+    upper: t.List[float]
+
+
 @dataclass_json
 @dataclass
 class ReserveMarket(object):
     price: t.List[float]
     offer_cap: float
-    utilization: Utilization
+    utilization: t.Union[Utilization, TimeSeriesUtilization]
+    obligation: t.Optional[t.List[float]] = opt_field()
 
 @dataclass_json
 @dataclass
